@@ -63,7 +63,7 @@ do
    end
 
    process_tree_args = function(args, project_dir)
-   
+
       if args.global then
          cfg.local_by_default = false
       end
@@ -85,7 +85,7 @@ do
             replace_tree(args, root_dir)
          end
       elseif args["local"] then
-         if not cfg.home_tree then
+         if fs.is_superuser() then
             return nil, "The --local flag is meant for operating in a user's home directory.\n"..
                "You are running as a superuser, which is intended for system-wide operation.\n"..
                "To force using the superuser's home, use --tree explicitly."
@@ -165,6 +165,7 @@ end
 -- @param exitcode number: the exitcode to use
 local function die(message, exitcode)
    assert(type(message) == "string", "bad error, expected string, got: " .. type(message))
+   assert(exitcode == nil or type(exitcode) == "number", "bad error, expected number, got: " .. type(exitcode) .. " - " .. tostring(exitcode))
    util.printerr("\nError: "..message)
 
    local ok, err = xpcall(util.run_scheduled_functions, error_handler)
@@ -179,7 +180,7 @@ end
 local function search_lua_in_path(lua_version, verbose)
    local path_sep = (package.config:sub(1, 1) == "\\" and ";" or ":")
    local all_tried = {}
-   for bindir in os.getenv("PATH"):gmatch("[^"..path_sep.."]+") do
+   for bindir in (os.getenv("PATH") or ""):gmatch("[^"..path_sep.."]+") do
       local parentdir = dir.path((bindir:gsub("[\\/][^\\/]+[\\/]?$", "")))
       local detected, tried = util.find_lua(parentdir, lua_version)
       if detected then
@@ -221,7 +222,7 @@ do
          end
          return nil
       end
-   
+
       local function find_default_lua_version(args, project_dir)
          if hardcoded.FORCE_CONFIG then
             return nil
@@ -231,8 +232,8 @@ do
          if project_dir then
             table.insert(dirs, dir.path(project_dir, ".luarocks"))
          end
-         if cfg.home_tree then
-            table.insert(dirs, dir.path(cfg.home_tree, ".luarocks"))
+         if cfg.homeconfdir then
+            table.insert(dirs, cfg.homeconfdir)
          end
          table.insert(dirs, cfg.sysconfdir)
          for _, d in ipairs(dirs) do
@@ -250,7 +251,7 @@ do
          end
          return nil
       end
-   
+
       local function find_version_from_config(dirname)
          return fun.find(util.lua_versions("descending"), function(v)
             if util.exists(dir.path(dirname, ".luarocks", "config-"..v..".lua")) then
@@ -258,12 +259,12 @@ do
             end
          end)
       end
-   
+
       local function detect_lua_via_args(args, project_dir)
          local lua_version = args.lua_version
                              or find_default_lua_version(args, project_dir)
                              or (project_dir and find_version_from_config(project_dir))
-      
+
          if args.lua_dir then
             local detected, err = util.find_lua(args.lua_dir, lua_version)
             if not detected then
@@ -274,7 +275,7 @@ do
             end
             return detected
          end
-      
+
          if lua_version then
             local detected = search_lua_in_path(lua_version)
             if detected then
@@ -284,10 +285,10 @@ do
                lua_version = lua_version,
             }
          end
-         
+
          return {}
       end
-      
+
       detect_config_via_args = function(args)
          local project_dir, given = find_project_dir(args.project_tree)
          local detected = detect_lua_via_args(args, project_dir)
@@ -304,15 +305,15 @@ do
          return detected
       end
    end
-   
+
    init_config = function(args)
       local detected = detect_config_via_args(args)
-   
+
       local ok, err = cfg.init(detected, util.warning)
       if not ok then
          return nil, err
       end
-      
+
       return (detected.lua_dir ~= nil)
    end
 end
@@ -337,7 +338,7 @@ local function use_to_fix_location(key)
    return buf
 end
 
-local function get_config_text(cfg)
+local function get_config_text(cfg)  -- luacheck: ignore 431
    local deps = require("luarocks.deps")
 
    local libdir_ok = deps.check_lua_libdir(cfg.variables)
@@ -642,7 +643,7 @@ function cmd.run_command(description, commands, external_namespace, ...)
    end
 
    -- if running as superuser, use system cache dir
-   if not cfg.home_tree then
+   if fs.is_superuser() then
       cfg.local_cache = dir.path(fs.system_cache_dir(), "luarocks")
    end
 
