@@ -14,7 +14,8 @@ local builtin = require("luarocks.build.builtin")
 local deplocks = require("luarocks.deplocks")
 
 --- Generate a function that matches dep queries against the manifest,
--- taking into account rocks_provided, the blacklist and the lockfile.
+-- taking into account rocks_provided, the list of versions to skip,
+-- and the lockfile.
 -- @param deps_mode "one", "none", "all" or "order"
 -- @param rocks_provided a one-level table mapping names to versions,
 -- listing rocks to consider provided by the VM
@@ -22,18 +23,18 @@ local deplocks = require("luarocks.deplocks")
 -- by this Lua implementation for the given dependency.
 -- @param depskey key to use when matching the lockfile ("dependencies",
 -- "build_dependencies", etc.)
--- @param blacklist a two-level table mapping names to versions to boolean,
--- listing rocks to not match
+-- @param skip_set a two-level table mapping names to versions to
+-- boolean, listing rocks that should not be matched
 -- @return function(dep): {string}, {string:string}, string, boolean
 -- * array of matching versions
 -- * map of versions to locations
 -- * version matched via lockfile if any
 -- * true if rock matched via rocks_provided
-local function prepare_get_versions(deps_mode, rocks_provided, depskey, blacklist)
+local function prepare_get_versions(deps_mode, rocks_provided, depskey, skip_set)
    assert(type(deps_mode) == "string")
    assert(type(rocks_provided) == "table")
    assert(type(depskey) == "string")
-   assert(type(blacklist) == "table" or blacklist == nil)
+   assert(type(skip_set) == "table" or skip_set == nil)
 
    return function(dep)
       local versions, locations
@@ -47,26 +48,23 @@ local function prepare_get_versions(deps_mode, rocks_provided, depskey, blacklis
          end
          versions, locations = manif.get_versions(dep, deps_mode)
       end
-      
-      if blacklist and blacklist[dep.name] then
-         local orig_versions = versions
-         versions = {}
-         for _, v in ipairs(orig_versions) do
-            if not blacklist[dep.name][v] then
-               table.insert(versions, v)
+
+      if skip_set and skip_set[dep.name] then
+         for i = #versions, 1, -1 do
+            local v = versions[i]
+            if skip_set[dep.name][v] then
+               table.remove(versions, i)
             end
          end
       end
-      
+
       local lockversion = deplocks.get(depskey, dep.name)
-   
+
       return versions, locations, lockversion, provided ~= nil
    end
 end
 
 --- Attempt to match a dependency to an installed rock.
--- @param blacklist table: Versions that can't be accepted. Table where keys
--- are program versions and values are 'true'.
 -- @param get_versions a getter function obtained via prepare_get_versions
 -- @return (string, string, table) or (nil, nil, table):
 -- 1. latest installed version of the rock matching the dependency
@@ -81,9 +79,9 @@ end
 local function match_dep(dep, get_versions)
    assert(type(dep) == "table")
    assert(type(get_versions) == "function")
-   
+
    local versions, locations, lockversion, provided = get_versions(dep)
-   
+
    local latest_version
    local latest_vstring
    for _, vstring in ipairs(versions) do
@@ -95,7 +93,7 @@ local function match_dep(dep, get_versions)
          end
       end
    end
-   
+
    if lockversion and not locations[lockversion] then
       local latest_matching_msg = ""
       if latest_vstring and latest_vstring ~= lockversion then
@@ -104,7 +102,7 @@ local function match_dep(dep, get_versions)
       util.printout("Forcing " .. dep.name .. " to pinned version " .. lockversion .. latest_matching_msg)
       return nil, nil, queries.new(dep.name, dep.namespace, lockversion)
    end
-   
+
    return latest_vstring, locations[latest_vstring], dep, provided
 end
 
@@ -113,7 +111,7 @@ local function match_all_deps(dependencies, get_versions)
    assert(type(get_versions) == "function")
 
    local matched, missing, no_upgrade = {}, {}, {}
-   
+
    for _, dep in ipairs(dependencies) do
       local found, _, provided
       found, _, dep, provided = match_dep(dep, get_versions)
@@ -135,7 +133,7 @@ end
 --- Attempt to match dependencies of a rockspec to installed rocks.
 -- @param dependencies table: The table of dependencies.
 -- @param rocks_provided table: The table of auto-provided dependencies.
--- @param blacklist table or nil: Program versions to not use as valid matches.
+-- @param skip_set table or nil: Program versions to not use as valid matches.
 -- Table where keys are program names and values are tables where keys
 -- are program versions and values are 'true'.
 -- @param deps_mode string: Which trees to check dependencies for
@@ -145,13 +143,13 @@ end
 -- parsed as tables; and a table of "no-upgrade" missing dependencies
 -- (to be used in plugin modules so that a plugin does not force upgrade of
 -- its parent application).
-function deps.match_deps(dependencies, rocks_provided, blacklist, deps_mode)
+function deps.match_deps(dependencies, rocks_provided, skip_set, deps_mode)
    assert(type(dependencies) == "table")
    assert(type(rocks_provided) == "table")
-   assert(type(blacklist) == "table" or blacklist == nil)
+   assert(type(skip_set) == "table" or skip_set == nil)
    assert(type(deps_mode) == "string")
 
-   local get_versions = prepare_get_versions(deps_mode, rocks_provided, "dependencies", blacklist)
+   local get_versions = prepare_get_versions(deps_mode, rocks_provided, "dependencies", skip_set)
    return match_all_deps(dependencies, get_versions)
 end
 
@@ -169,7 +167,7 @@ end
 -- @param version string: package version.
 -- @param dependencies table: array of dependencies.
 -- @param deps_mode string: Which trees to check dependencies for
--- @param rocks_provided table: A table of auto-dependencies provided 
+-- @param rocks_provided table: A table of auto-dependencies provided
 -- by this Lua implementation for the given dependency.
 -- "one" for the current default tree, "all" for all trees,
 -- "order" for all trees with priority >= the current default, "none" for no trees.
@@ -179,7 +177,7 @@ function deps.report_missing_dependencies(name, version, dependencies, deps_mode
    assert(type(dependencies) == "table")
    assert(type(deps_mode) == "string")
    assert(type(rocks_provided) == "table")
-   
+
    if deps_mode == "none" then
       return
    end
@@ -270,7 +268,7 @@ local function check_supported_platforms(rockspec)
          return nil, "This rockspec for "..rockspec.package.." does not support "..plats.." platforms."
       end
    end
-   
+
    return true
 end
 
@@ -278,7 +276,8 @@ end
 -- Packages are installed using the LuaRocks "install" command.
 -- Aborts the program if a dependency could not be fulfilled.
 -- @param rockspec table: A rockspec in table format.
--- @param depskey string: Rockspec key to fetch to get dependency table.
+-- @param depskey string: Rockspec key to fetch to get dependency table
+-- ("dependencies", "build_dependencies", etc.).
 -- @param deps_mode string
 -- @param verify boolean
 -- @param deplock_dir string: dirname of the deplock file
@@ -295,7 +294,7 @@ function deps.fulfill_dependencies(rockspec, depskey, deps_mode, verify, deplock
    local name = rockspec.name
    local version = rockspec.version
    local rocks_provided = rockspec.rocks_provided
-      
+
    local ok, filename, err = deplocks.load(name, deplock_dir or ".")
    if filename then
       util.printout("Using dependencies pinned in lockfile: " .. filename)
@@ -530,7 +529,7 @@ local function check_external_dependency(name, ext_files, vars, mode, cache)
    local err_files = {program = {}, header = {}, library = {}}
 
    local dirs = get_external_deps_dirs(mode)
-   
+
    local prefixes
    if vars[name .. "_DIR"] then
       prefixes = { vars[name .. "_DIR"] }
@@ -539,7 +538,7 @@ local function check_external_dependency(name, ext_files, vars, mode, cache)
    else
       prefixes = cfg.external_deps_dirs
    end
-   
+
    for _, prefix in ipairs(prefixes) do
       prefix = resolve_prefix(prefix, dirs)
       if cfg.is_platform("mingw32") and name == "LUA" then
@@ -556,7 +555,7 @@ local function check_external_dependency(name, ext_files, vars, mode, cache)
          return true
       end
    end
-   
+
    return nil, err_dirname, err_testfile, err_files
 end
 
@@ -575,7 +574,7 @@ end
 -- nil and an error message if any test failed.
 function deps.check_external_deps(rockspec, mode)
    assert(rockspec:type() == "rockspec")
-   
+
    if not rockspec.external_dependencies then
       rockspec.external_dependencies = builtin.autodetect_external_dependencies(rockspec.build)
    end
@@ -587,7 +586,7 @@ function deps.check_external_deps(rockspec, mode)
       local ok, err_dirname, err_testfile, err_files = check_external_dependency(name, ext_files, rockspec.variables, mode)
       if not ok then
          local lines = {"Could not find "..err_testfile.." file for "..name}
-      
+
          local err_paths = {}
          for _, err_file in ipairs(err_files[err_testfile]) do
             if not err_paths[err_file] then
@@ -595,10 +594,10 @@ function deps.check_external_deps(rockspec, mode)
                table.insert(lines, "  No file "..err_file)
             end
          end
-      
+
          table.insert(lines, "You may have to install "..name.." in your system and/or pass "..name.."_DIR or "..name.."_"..err_dirname.." to the luarocks command.")
          table.insert(lines, "Example: luarocks install "..rockspec.name.." "..name.."_DIR=/usr/local")
-      
+
          return nil, table.concat(lines, "\n"), "dependency"
       end
    end
@@ -700,7 +699,7 @@ function deps.check_lua_incdir(vars)
    if vars.LUA_INCDIR then
       return lua_h_exists(vars.LUA_INCDIR, cfg.lua_version)
    end
-   
+
    if vars.LUA_DIR then
       vars.LUA_INCDIR = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, ljv)
       if vars.LUA_INCDIR then
