@@ -132,7 +132,7 @@ function fs_lua.is_tool_available(tool_cmd, tool_name, arg)
 
    if ok then
       return true
-   else   
+   else
       local msg = "'%s' program not found. Make sure %s is installed and is available in your PATH " ..
                   "(or you may want to edit the 'variables.%s' value in file '%s')"
       return nil, msg:format(tool_cmd, tool_name, tool_name:upper(), cfg.config_files.nearest)
@@ -215,7 +215,7 @@ function fs_lua.modules(at)
 
    local modules = {}
    local is_duplicate = {}
-   for _, path in ipairs(paths) do
+   for _, path in ipairs(paths) do  -- luacheck: ignore 421
       local files = fs.list_dir(path)
       for _, filename in ipairs(files or {}) do
          if filename:match("%.[lL][uU][aA]$") then
@@ -535,7 +535,7 @@ end
 --- Internal implementation function for fs.dir.
 -- Yields a filename on each iteration.
 -- @param at string: directory to list
--- @return nil or (nil and string): an error message on failure 
+-- @return nil or (nil and string): an error message on failure
 function fs_lua.dir_iterator(at)
    local pok, iter, arg = pcall(lfs.dir, at)
    if not pok then
@@ -705,9 +705,9 @@ local redirect_protocols = {
    https = luasec_ok and https,
 }
 
-local function request(url, method, http, loop_control)
+local function request(url, method, http, loop_control)  -- luacheck: ignore 431
    local result = {}
-   
+
    if cfg.verbose then
       print(method, url)
    end
@@ -805,7 +805,7 @@ end
 -- via the HTTP Last-Modified header if the full download is needed.
 -- @return (boolean | (nil, string, string?)): True if successful, or
 -- nil, error message and optionally HTTPS error in case of errors.
-local function http_request(url, filename, http, cache)
+local function http_request(url, filename, http, cache)  -- luacheck: ignore 431
    if cache then
       local status = read_timestamp(filename..".status")
       local timestamp = read_timestamp(filename..".timestamp")
@@ -824,7 +824,7 @@ local function http_request(url, filename, http, cache)
             end
          end
 
-         local result, status, headers, err = request(url, "HEAD", http)
+         local result, status, headers, err = request(url, "HEAD", http)  -- luacheck: ignore 421
          if not result then
             return fail_with_status(filename, status, headers)
          end
@@ -1034,6 +1034,10 @@ function fs_lua.current_user()
    return posix.getpwuid(posix.geteuid()).pw_name
 end
 
+function fs_lua.is_superuser()
+   return false
+end
+
 -- This call is not available on all systems, see #677
 if posix.mkdtemp then
 
