@@ -15,8 +15,11 @@ local util = require("luarocks.util")
 -- See http://lua-users.org/lists/lua-l/2013-11/msg00367.html
 local _prefix = "type NUL && "
 local _popen, _execute = io.popen, os.execute
+
+-- luacheck: push globals io os
 io.popen = function(cmd, ...) return _popen(_prefix..cmd, ...) end
 os.execute = function(cmd, ...) return _execute(_prefix..cmd, ...) end
+-- luacheck: pop
 
 --- Annotate command string for quiet execution.
 -- @param cmd string: A command-line string.
@@ -38,7 +41,7 @@ end
 local function split_root(pathname)
    local drive = ""
    local root = ""
-   local rest = ""
+   local rest
 
    local unquoted = pathname:match("^['\"](.*)['\"]$")
    if unquoted then
@@ -163,12 +166,18 @@ function win32.wrap_script(script, target, deps_mode, name, version, ...)
       "package.path="..util.LQ(lpath..";").."..package.path",
       "package.cpath="..util.LQ(lcpath..";").."..package.cpath",
    }
+
+   local remove_interpreter = false
    if target == "luarocks" or target == "luarocks-admin" then
+      if cfg.is_binary then
+         remove_interpreter = true
+      end
       luainit = {
          "package.path="..util.LQ(package.path),
          "package.cpath="..util.LQ(package.cpath),
       }
    end
+
    if name and version then
       local addctx = "local k,l,_=pcall(require,'luarocks.loader') _=k " ..
                      "and l.add_context('"..name.."','"..version.."')"
@@ -182,6 +191,11 @@ function win32.wrap_script(script, target, deps_mode, name, version, ...)
       script and fs.Qb(script) or "%I%",
       ...
    }
+   if remove_interpreter then
+      table.remove(argv, 1)
+      table.remove(argv, 1)
+      table.remove(argv, 1)
+   end
 
    wrapper:write("@echo off\r\n")
    wrapper:write("setlocal\r\n")
@@ -203,7 +217,7 @@ function win32.is_actual_binary(name)
    return false
 end
 
-function win32.copy_binary(filename, dest) 
+function win32.copy_binary(filename, dest)
    local ok, err = fs.copy(filename, dest)
    if not ok then
       return nil, err
@@ -325,13 +339,22 @@ function win32.make_temp_dir(name_pattern)
 end
 
 function win32.tmpname()
-   return os.getenv("TMP")..os.tmpname()
+   local name = os.tmpname()
+   local tmp = os.getenv("TMP")
+   if tmp and name:sub(1, #tmp) ~= tmp then
+      name = (tmp .. "\\" .. name):gsub("\\+", "\\")
+   end
+   return name
 end
 
 function win32.current_user()
    return os.getenv("USERNAME")
 end
 
+function win32.is_superuser()
+   return false
+end
+
 function win32.export_cmd(var, val)
    return ("SET %s=%s"):format(var, val)
 end
